'''\
wrap lines to a constant width.

This module's only public function is 'wrapper', which takes a string, 
a maximum line length, and justification mode, and returns a string with
all lines less then the maximum line length, justified according to the
third argument.

Note that whitespace is used only to seperate word, and is not significant:
in particular, this function does *not* assume that a blank line signifies
end of paragraph. Any such paragraph division must be done as pre-processing.

This module also exports the constants 'LEFT', 'RIGHT', 'CENTER' and 'FILL',
which signify justification.
'''
import re, string, operator

LEFT, RIGHT, CENTER, FILL = 0, 1, 2, 3
_whitespace=re.compile('[ \t\n\r]+')

def wrapper(text, line_len, justification=None):
	'''\
	wrapp text at given line breaks.

	Arguments:

	'text' -- the text to be justified.

	'line_len' -- maximum line length

	'justification' -- type of justification (default is 'LEFT')

	Return value: the text, wrapped and justified.
	'''
	word_list=filter(None, _whitespace.split(text))
	lines=_split_lines(word_list, line_len)
	lines=map(_justify, lines, 
	                    [line_len]*len(lines),
	                    [justification]*len(lines))
	return string.join(lines, '\n')+'\n'

def _justify(line, line_len, justification):
	if not justification: return string.join(line, ' ')
	if justification==RIGHT:
		s=string.join(line, ' ')
		n=line_len-len(s)
		return ' '*n+s
	if justification==CENTER:
		s=string.join(line, ' ')
		n=line_len-len(s)
		return ' '*(n/2)+s
	if justification==FILL:
		orig_len=reduce(operator.add, map(len, line))
		n=len(line)
		if n==1: return line[0]
		space_width=(line_len-orig_len)/(n-1.0)
		s=''
		owed_spaces=0.0
		for word in line:
			s=s+word
			owed_spaces=owed_spaces+space_width
			s=s+' '*int(owed_spaces)
			owed_spaces=owed_spaces-int(owed_spaces)
		return s
	raise ValueError, 'unknown justification type'

def _split_lines(word_list, line_len):
	lines=[]
	current_line=[]
	current_len=0
	for word in word_list:
		if current_len+1+len(word)>line_len:
			lines.append(current_line)
			current_line, current_len = [], 0
		current_line.append(word)
		current_len=current_len+len(word)+1
	if current_line: lines.append(current_line)
	return lines

_sample_text='''\
          Return a tuple containing all the subgroups of the match, from
          1 up to however many groups are in the pattern. Groups that did
          not participate in the match have values of None.
'''*8

def _test():
	import sys
	for just in (LEFT, RIGHT, CENTER, FILL):
		sys.stdout.write('-'*50+'\n')
		sys.stdout.write(wrapper(_sample_text, 50, just))
		sys.stdout.write('-'*50+'\n')

if __name__=='__main__': _test()
